import os
import unittest
from unittest.mock import patch

import jupyterlab
import jupyterlab_server
import pytest

from aext_shared.config import (
    ConfigValueFactory,
    Environment,
    get_config,
    get_environment_v2,
)


@pytest.fixture
def get_os_info_mock_response():
    return {
        "name": "AAAAmacOS-14.4.1-arm64-arm-64bit",
        "system": "Darwin",
        "version": "Darwin Kernel Version 23.4.0: Fri Mar 15 00:10:42 PDT 2024; root:xnu-10063.101.17~1/RELEASE_ARM64_T6000",
        "release": "23.4.0",
    }


@pytest.fixture()
def get_os_info_mock(get_os_info_mock_response):
    with patch("aext_shared.config.get_os_info") as m:
        m.return_value = get_os_info_mock_response
        yield m


@pytest.fixture
def default_config(get_os_info_mock_response):
    return {
        "heap": {"clientId": "758475466"},
        "datadog": {"clientToken": "pub2b6467ca1f0ee597585d8a8bf43d523f", "service": "nb.anaconda.cloud"},
        "featureFlag": {
            "clientId": "6408b2b7e550011343183538",
            "overrides": {},
        },
        "cloud": {
            "url": "https://anaconda.cloud/api",
            "staticContent": "https://static.anaconda.cloud/shared",
        },
        "org": {
            "url": "https://notebooks.anaconda.org",
        },
        "notebooks": {"url": "https://nb.anaconda.cloud"},
        "environment": Environment.development,
        "environment_v2": Environment.local_development,
        "jupyterlab": {"server": jupyterlab_server.__version__, "client": jupyterlab.__version__},
        "assistant": {
            "environmentType": "cloud-notebooks-prod",
            "apiUrl": "https://assistant.anaconda.cloud",
        },
        "platform": get_os_info_mock_response,
    }


@patch.dict(os.environ, {"LOCAL_DEVELOPMENT": "1"}, clear=True)
def test_get_default_config(default_config, get_os_info_mock):
    conf = get_config()
    assert conf == default_config


@patch.dict(
    os.environ,
    {
        "PYTHONANYWHERE_SITE": "https://nb.anaconda.cloud",
        "ANACONDA_NOTEBOOKS_URL": "https://nb.anaconda.cloud",
    },
    clear=True,
)
@patch.dict(os.environ, {"LOCAL": "true"}, clear=True)
def test_get_config_with_production_environment(default_config):
    config = get_config()
    assert config["environment"] == "local"
    assert config["environment_v2"] == "local_production"


@patch.dict(
    os.environ,
    {"DEBUG_AEXT_SHARED_LOCAL": "true", "LOCAL_PRODUCTION": "true"},
    clear=True,
)
def test_local_config(default_config, get_os_info_mock, get_os_info_mock_response):
    config = get_config()
    expected = default_config
    expected["heap"]["clientId"] = None
    expected["datadog"]["clientToken"] = None
    expected["datadog"]["service"] = None
    expected["featureFlag"]["clientId"] = None
    expected["featureFlag"]["overrides"] = {
        "monorepo-assistant": True,
        "monorepo-panel": True,
        "monorepo-panel-show-publish-button": False,
        "monorepo-share-notebooks": True,
        "monorepo-show-project-file-status": False,
        "monorepo-show-anaconda-toolbox": False,
        "panel-sign-out-jupyterlab": True,
        "share-notebook-use-projects-v2": True,
        "sharing-show-preview": True,
    }
    expected["environment"] = Environment.local
    expected["assistant"] = {
        "environmentType": "local-notebooks-prod",
        "apiUrl": "https://assistant.anaconda.cloud",
    }
    expected["platform"] = get_os_info_mock_response
    expected["environment"] = Environment.local
    expected["environment_v2"] = Environment.local_production
    assert config == expected


class TestGetEnvironmentV2(unittest.TestCase):

    @patch.dict(os.environ, {"GITHUB_ACTIONS": "1"}, clear=True)
    def test_github_actions(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.github_actions)

    @patch.dict(os.environ, {"LOCAL_DEVELOPMENT": "1"}, clear=True)
    def test_local_development(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.local_development)

    @patch.dict(os.environ, {"LOCAL_PRODUCTION": "1"}, clear=True)
    def test_local_production(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.local_production)

    @patch.dict(os.environ, {"PYTHONANYWHERE_SITE": "http://nb.anaconda.cloud"}, clear=True)
    def test_cloud_production(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_production)

    @patch.dict(os.environ, {"PYTHONANYWHERE_SITE": "http://tulz.pa-dev.com"}, clear=True)
    def test_cloud_development(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_development)

    @patch.dict(os.environ, {}, clear=True)
    def test_default_env(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_production)


@pytest.mark.parametrize(
    "env, expected",
    [
        (Environment.local, True),
        (Environment.local_production, True),
        (Environment.local_development, True),
        (Environment.cloud_production, False),
        (Environment.cloud_development, False),
    ],
)
def test_environment_is_local(env, expected):
    assert Environment.is_local(env) == expected


@pytest.mark.parametrize(
    "env, expected",
    [
        (Environment.local, False),
        (Environment.local_production, False),
        (Environment.local_development, False),
        (Environment.cloud_production, True),
        (Environment.cloud_development, True),
    ],
)
def test_environment_is_cloud(env, expected):
    assert Environment.is_cloud(env) == expected
